var RouteController, bodyParser, ensureSlash, exports, methodOverride, multer, routeHandler, routeMapperAdapter, traverse, validateMethod, _;

_ = require('lodash');

routeHandler = require('./routeHandler');

validateMethod = require('./helpers/validateMethod');

traverse = require('./traverse');

RouteController = require('./routeController');

methodOverride = require('method-override');

bodyParser = require('body-parser');

multer = require('multer');

ensureSlash = function(path) {
  if (path[0] !== '/') {
    path = "/" + path;
  }
  if (path[path.length - 1] !== '/') {
    path = "" + path + "/";
  }
  return path;
};

routeMapperAdapter = function(express) {
  express.use(methodOverride());
  express.use(bodyParser.urlencoded({
    extended: true
  }));
  express.use(bodyParser.json({
    type: 'application/vnd.api+json'
  }));
  express.use(bodyParser.json());
  express.use(multer());
  express.routeMapper = function(options) {
    var baseHost, baseRoute, conditionalInheritance, debug, key, routes, server, _results;
    if (!options) {
      throw new Error('missing options');
    }
    debug = options.debug || false;
    routes = options.routes;
    baseRoute = options.base;
    baseHost = options.host;
    server = options.server;
    if (options.mvc && typeof options.mvc === 'string') {
      traverse(options.mvc, express.routeMapper, options);
      if (!routes) {
        return express;
      }
    }
    if (!routes) {
      throw new Error('missing routes');
    }
    if (baseRoute == null) {
      baseRoute = '';
    }
    conditionalInheritance = null;
    _.sortBy(routes, function(key) {
      return typeof key === 'function';
    });
    _results = [];
    for (key in routes) {
      _results.push((function(key, baseRoute, routes, options, server, debug) {
        var route, routePath;
        switch (typeof routes[key]) {
          case "object":
            routePath = baseRoute + key;
            return express.routeMapper({
              config: options.config,
              base: routePath,
              routes: routes[key],
              baseHost: baseHost,
              server: server,
              debug: debug,
              parent: {
                conditions: conditionalInheritance,
                debug: debug
              }
            });
          case "function":
            validateMethod(key);
            route = routes[key];
            if (options.parent && options.parent.conditions) {
              route.conditions = _.assign(options.parent.conditions, route.conditions);
            }
            conditionalInheritance = route.conditions;
            return routeHandler.apply(express, [
              new RouteController(key, ensureSlash(baseRoute), route, options, {
                baseHost: baseHost,
                server: server,
                debug: debug,
                express: express
              })
            ]);
        }
      })(key, baseRoute, routes, options, server, debug));
    }
    return _results;
  };
  return express;
};

exports = module.exports = routeMapperAdapter;
