#==============================================================================
# Deps
#==============================================================================
_ = require 'lodash'
routeHandler = require './routeHandler' #route setup
validateMethod = require './helpers/validateMethod' #validate http verb
traverse = require './traverse'
RouteController = require './routeController'
methodOverride = require('method-override')
bodyParser = require('body-parser')
multer = require('multer')




ensureSlash = (path) ->
  if path[0] isnt '/'
    path = "/#{path}"
  if path[path.length - 1] isnt '/'
    path = "#{path}/"
  path

#==============================================================================
# Express Adapter: Route Mapper
#==============================================================================
routeMapperAdapter = (express) ->
  express.use(methodOverride())
  express.use(bodyParser.urlencoded({ extended: true }))
  express.use(bodyParser.json({ type: 'application/vnd.api+json' }))
  express.use(bodyParser.json())
  express.use(multer())

  express.routeMapper = (options) ->
    return throw new Error 'missing options' unless options
    debug = options.debug or false
    routes = options.routes
    baseRoute = options.base
    baseHost = options.host
    server = options.server
    if options.mvc and typeof(options.mvc) is 'string'
      traverse options.mvc, express.routeMapper, options
      return express unless routes


    #throw error is missing routes
    return throw new Error 'missing routes' unless routes
    baseRoute ?= ''
    conditionalInheritance = null

    _.sortBy routes, (key) ->
      typeof key is 'function'

    # loop routes
    for key of routes
      do (key, baseRoute, routes, options, server, debug) ->
        switch typeof routes[key]
          when "object"
            #build route path
            routePath = baseRoute + key

            express.routeMapper
              config: options.config #route setting
              base: routePath
              routes: routes[key] #route callback
              baseHost: baseHost
              server: server
              debug: debug
              parent:
                conditions: conditionalInheritance #inherited object
                debug: debug

          when "function"
            #validate http verb
            validateMethod key
            #route callback
            route = routes[key]
            #if inherited, merge
            if options.parent and options.parent.conditions
              route.conditions = _.assign options.parent.conditions, route.conditions
            conditionalInheritance = route.conditions

            #handle route setup
            routeHandler.apply express, [new RouteController key, ensureSlash(baseRoute), route, options,
              baseHost: baseHost
              server: server
              debug: debug
              express: express
            ]

  return express

exports = module.exports = routeMapperAdapter
